#include "distro_arch_series_filter.h"
#include "launchpad.h"
#include <iostream>
#include <nlohmann/json.hpp>

distro_arch_series_filter::distro_arch_series_filter()
    : lp(nullptr) {}

distro_arch_series_filter::~distro_arch_series_filter() {}

std::optional<distro_arch_series_filter> distro_arch_series_filter::parse(const std::string& json_data) {
    distro_arch_series_filter f;
    f.parse_json(json_data);
    return f;
}

void distro_arch_series_filter::parse_json(const std::string& json_data) {
    if (json_data.empty()) {
        std::cerr << "Error: Empty JSON data for distro_arch_series_filter::parse_json." << std::endl;
        return;
    }

    try {
        auto data = nlohmann::json::parse(json_data);

        auto get_str = [&](const std::string& key, std::string& field) {
            if (data.contains(key) && data[key].is_string())
                field = data[key].get<std::string>();
        };

        get_str("creator_link", creator_link);
        get_str("date_created", date_created);
        get_str("date_last_modified", date_last_modified);
        get_str("distroarchseries_link", distroarchseries_link);
        get_str("http_etag", http_etag);
        get_str("packageset_link", packageset_link);
        get_str("resource_type_link", resource_type_link);
        get_str("self_link", self_link);
        get_str("sense", sense);
        get_str("web_link", web_link);

    } catch (const nlohmann::json::parse_error& e) {
        std::cerr << "JSON parse error in distro_arch_series_filter::parse_json: " << e.what() << std::endl;
    } catch (const std::exception& e) {
        std::cerr << "Unexpected error during JSON parsing: " << e.what() << std::endl;
    }
}

void distro_arch_series_filter::set_lp(launchpad* lp_ptr) {
    lp = lp_ptr;
}

std::optional<distro_arch_series_filter> distro_arch_series_filter::get() const {
    if (self_link.empty()) return std::nullopt;
    auto resp = lp->api_get(self_link);
    if (!resp) return std::nullopt;
    auto f = parse(resp.value());
    if (f) f->set_lp(lp);
    return f;
}

bool distro_arch_series_filter::patch(const nlohmann::json& data) const {
    if (self_link.empty()) return false;
    auto resp = lp->api_patch(self_link, data);
    return resp.has_value();
}

bool distro_arch_series_filter::put(const nlohmann::json& data) const {
    if (self_link.empty()) return false;
    // Emulate PUT with ws.op=replace
    std::string endpoint = self_link + "?ws.op=replace";
    auto resp = lp->api_patch(endpoint, data);
    return resp.has_value();
}
