// Copyright (C) 2024 Simon Quigley <tsimonq2@ubuntu.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#ifndef PERSON_H
#define PERSON_H

#include <string>
#include <optional>
#include <nlohmann/json.hpp>

#ifndef LAUNCHPAD_API
#ifdef BUILDING_LAUNCHPAD
  #define LAUNCHPAD_API __attribute__((visibility("default")))
#else
  #define LAUNCHPAD_API
#endif
#endif

class archive;
class distribution;
class launchpad;

class LAUNCHPAD_API person {
public:
    person();
    ~person();

    // Static parse method
    static std::optional<person> parse(const std::string& json_data);

    // Get a PPA by name
    std::optional<archive> getPPAByName(distribution& distribution, const std::string& name_);

    // Public member variables
    std::string name;
    std::string realname;
    std::string web_link;
    std::string email;
    std::string description;
    std::string visibility;
    std::string self_link;

    // Additional read-only fields
    std::string ppas_collection_link;
    std::string preferred_email_address_link;

    void set_lp(launchpad* lp_ptr);

private:
    launchpad* lp; // store pointer to lp

    // JSON parsing utility
    void parse_json(const std::string& json_data);
};

#endif // PERSON_H
